﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NDesk.Options;

namespace xml2xlsx
{
    class Program
    {
        static void ShowHelp(string Message, OptionSet OptionSet)
        {
            if (!String.IsNullOrEmpty(Message))
                Console.WriteLine(Message + Environment.NewLine);

            Console.WriteLine("Usage: " + Environment.NewLine
                + System.AppDomain.CurrentDomain.FriendlyName
                + " -f XML file -e root element [-n elements with numeric values] [-i ignored elements] [-c maximum rows in spreadsheet] [-t add current time to XLSX file name] [-v be more verbose] [-u url [-x don't use proxy] [-s username] [-p password]]");

            Console.WriteLine();
            Console.WriteLine("Arguments:");
            OptionSet.WriteOptionDescriptions(Console.Out);

            Console.WriteLine(Environment.NewLine
                + "This program contains following components:" + Environment.NewLine
                + "NDesk.Options parser: http://ndesk.org/Options" + Environment.NewLine
                + "ExtremeML library: http://www.extrememl.com");

            Environment.Exit(-1);
        }

        static void Main(string[] args)
        {
            AppArguments appArguments = new AppArguments();
            bool showHelp = false;

            var options = new OptionSet()
            {
                {"?|h|help", "Displays this help.",
                    v => showHelp = v != null},
                {"f|file=", "Path to XML file.",
                    v => appArguments.XmlFile = v},
                {"e|element=", "Root element in XML file.",
                    v => appArguments.RootElement = v},
                {"n|number_elements=", "Comma separated list of elements with numeric values. Values in these elements will be stored in the XLSX file as numeric values.",
                    v => appArguments.NumberElements = v.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries).ToList()},
                {"i|ignore_elements=", "Comma separated list of ignored elements in XML file.",
                    v => appArguments.IgnoreElements = v.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries).ToList()},
                {"c|sheet_count=", "Maximum saved elements to each spreadsheet.",
                    (int v) => appArguments.ElementsInSheet = v},
                {"t|add_time", "Add current time to XLSX file name.",
                    v => appArguments.AddTimeStamp = v != null},
                {"v|verbose", "Write element number to console.",
                    v => appArguments.Verbose = v != null},
                {"u|url=", "Remote XML file URL.",
                    v => appArguments.Url = v},
                {"x|noproxy", "Don't use proxy to download file.",
                    v => appArguments.NoProxy = v != null},
                {"s|username=", "Username to authenticate to remote server.",
                    v => appArguments.Username = v},
                {"p|password=", "Password to authenticate to remote server.",
                    v => appArguments.Password = v},
            };

            try
            {
                options.Parse(args);
            }
            catch (OptionException e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine();
                ShowHelp(null, options);
            }

            if (showHelp)
                ShowHelp(null, options);

            // Kontrola, zda jsou zadany povinne parametry.
            if (String.IsNullOrEmpty(appArguments.XmlFile))
                ShowHelp("Specify path to XML file.", options);

            if (String.IsNullOrEmpty(appArguments.RootElement))
                ShowHelp("Specify root element in XML file.", options);

            if (appArguments.ElementsInSheet < 0)
                ShowHelp("Parameter sheet_count must by positive number.", options);

            try
            {
                Xml2Xlsx xml2Xlsx = new Xml2Xlsx();
                xml2Xlsx.ConvertToXlsx(appArguments);
            }
            catch (Exception e)
            {
                if (appArguments.Verbose)
                {
                    Console.WriteLine(e.StackTrace);
                    Console.WriteLine();
                }
                Console.WriteLine(e.Message);
            }
        }
    }
}
