﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Xml.Linq;
using System.IO;
using System.Text;

namespace xml2xlsx
{
    class Xml2Xlsx
    {
        List<string> sheetColumns;
        int sheetRow;
        int sheetCounter;
        int elementCounter;

        public Xml2Xlsx()
        {
            sheetColumns = new List<string>();
            sheetRow = 0;
            sheetCounter = 1;
            elementCounter = 0;
        }

        // Source: http://stackoverflow.com/questions/2441673/reading-xml-with-xmlreader-in-c-sharp
        IEnumerable<XElement> XEnum(string xmlFile, string elementName)
        {
            using (XmlReader reader = XmlReader.Create(xmlFile))
            {
                //reader.MoveToContent();

                while (reader.Read())
                {
                    //Console.WriteLine(reader.Name);

                    if (reader.NodeType == XmlNodeType.Element && reader.Name == elementName)
                    {
                        //Console.WriteLine(reader.Name);

                        var st = reader.ReadSubtree();
                        st.Read();
                        XElement el = XNode.ReadFrom(st) as XElement;
                        st.Close();

                        if (el != null)
                        {
                            yield return el;
                        }
                    }
                }

                reader.Close();
            }
        }

        string GenerateFileName(AppArguments appArguments)
        {
            // Slozka, ve ktere je ulozen zdrojovy soubor.
            string sourceFolder = Path.GetDirectoryName(appArguments.XmlFile);

            // Aktualni cas.
            string timePart;

            if (appArguments.AddTimeStamp)
                timePart = DateTime.Now.ToString("-yyyyMMdd-HHmmss", System.Globalization.CultureInfo.InvariantCulture);
            else
                timePart = String.Empty;

            // Poradove cislo sesitu.
            string sheetNumber;

            if (appArguments.ElementsInSheet > 0)
            {
                sheetNumber = "-" + sheetCounter.ToString("000");
                sheetCounter++;
            }
            else
                sheetNumber = String.Empty;

            // Jmeno noveho XLSX souboru.
            string xlsxFileName = Path.GetFileNameWithoutExtension(appArguments.XmlFile) + timePart + sheetNumber + ".xlsx";

            // Slozeni cele cesty, pokud je k dispozici slozka zdojoveho souboru.
            if (!String.IsNullOrEmpty(sourceFolder))
                xlsxFileName = Path.Combine(sourceFolder, xlsxFileName);

            return xlsxFileName;
        }

        ExtremeML.Packaging.SpreadsheetDocumentWrapper CreateSpreadsheet(AppArguments appArguments)
        {
            string xlsxFileName = GenerateFileName(appArguments);
            var spreadsheet = ExtremeML.Packaging.SpreadsheetDocumentWrapper.Create(xlsxFileName);

            Console.WriteLine("Created file " + xlsxFileName);

            return spreadsheet;
        }

        public void ConvertToXlsx(AppArguments appArguments)
        {
            if (appArguments.Url.Length > 0)
                DownloadFile(appArguments);

            //return;
            // Kontrola, zda XML soubor existuje.
            if (!File.Exists(appArguments.XmlFile))
            {
                Console.WriteLine("XML file not found.");
                return;
            }

            // Vytvoreni noveho XLSX souboru.
            var spreadsheet = CreateSpreadsheet(appArguments);

            // Vlozeni noveho listu.
            var worksheet = spreadsheet.WorkbookPart.WorksheetParts.Add("Sheet1");

            // Vyhledani elementu se zadanym jmenem ve zdrojovem XML souboru.
            foreach (XElement xe in XEnum(appArguments.XmlFile, appArguments.RootElement))
            {
                //Console.WriteLine(xe);

                // Vraceny element musi mit vnoreny dalsi elementy.
                if (xe.HasElements)
                {
                    bool addNewRow = true;

                    //Console.WriteLine(xe);

                    if (appArguments.Verbose)
                        Console.WriteLine("Processing element " + (++elementCounter).ToString());

                    // Zpracovani vsech vnorenych elementu.
                    foreach (XNode node in xe.Nodes())
                    {
                        // Kontrola, zda je uzel typu XElement.
                        XElement element = node as XElement;
                        if (element == null)
                            continue;

                        //Console.WriteLine(node);

                        // Nacteni jmena uzlu.
                        string key = element.Name.LocalName;

                        if (String.IsNullOrEmpty(key))
                            continue;

                        // Kontrola, zda ma byt element preskocen.
                        if (appArguments.IgnoreElements != null
                            && appArguments.IgnoreElements.Contains(key))
                            continue;

                        // Inkrementace radku v XLSX souboru.
                        if (addNewRow)
                        {
                            addNewRow = false;
                            ++sheetRow;
                        }

                        // Hodnota uzlu.
                        string value = element.Value;

                        // Vyhledani sloupce, do ktereho bude v XLSX souboru
                        // ulozena hodnota uzlu.
                        int index = sheetColumns.IndexOf(key);

                        // Pokud uzel s timto jmenem jeste neni v XLSX souboru ulozen,
                        // do sesitu se doplni novy sloupec.
                        if (index == -1)
                        {
                            sheetColumns.Add(key);
                            index = sheetColumns.Count - 1;

                            worksheet.Worksheet.SetCellValue(new ExtremeML.Spreadsheet.Address.GridReference(0, index), key);
                        }

                        // Test, zda jde o cislo a ulozeni hodnoty do sesitu.
                        double d;

                        if (appArguments.NumberElements != null
                            && appArguments.NumberElements.Contains(key)
                            && Double.TryParse(value.Replace(",", "."),
                                System.Globalization.NumberStyles.Number,
                                System.Globalization.CultureInfo.InvariantCulture,
                                out d))
                            worksheet.Worksheet.SetCellValue(new ExtremeML.Spreadsheet.Address.GridReference(sheetRow, index), d);
                        else
                            worksheet.Worksheet.SetCellValue(new ExtremeML.Spreadsheet.Address.GridReference(sheetRow, index), value);
                    }

                    // Vytvoreni noveho sesitu, pokud je dosazeno limitu zaznamu na 1 sesit.
                    if (appArguments.ElementsInSheet > 0 && sheetRow >= appArguments.ElementsInSheet)
                    {
                        // Uzavreni stavajiciho.
                        spreadsheet.Dispose();

                        // Vytvoreni noveho.
                        spreadsheet = CreateSpreadsheet(appArguments);
                        worksheet = spreadsheet.WorkbookPart.WorksheetParts.Add("Sheet1");

                        sheetRow = 0;

                        // Doplneni nazvu sloupcu do noveho sesitu.
                        for (int i = 0; i < sheetColumns.Count; i++)
                        {
                            worksheet.Worksheet.SetCellValue(
                                new ExtremeML.Spreadsheet.Address.GridReference(0, i),
                                sheetColumns[i]);
                        }
                    }
                }
            }

            spreadsheet.Dispose();
        }

        void DownloadFile(AppArguments appArguments)
        {
            using (GzipWebClient webc = new GzipWebClient())
            {
                // Bez proxy.
                if (appArguments.NoProxy)
                    webc.Proxy = null;

                // Prihlaseni ke vzdalenemu serveru.
                if (appArguments.Username.Length > 0)
                    webc.Credentials = new System.Net.NetworkCredential(appArguments.Username, appArguments.Password);

                Uri uri = new Uri(appArguments.Url);

                webc.DownloadFile(uri, appArguments.XmlFile);
            }
        }
    }
}
